import urllib
import simplejson
import gnosis.xml.objectify
import gdata

import apikey

# --------------------------------------------------------------------
# Utilities

def load_url(url):
    response = urllib.urlopen(url)
    return response.read()

def load_url_appengine(url):
    from google.appengine.api import urlfetch
    response = urlfetch.fetch(url)
    return response.content

def load_json(url):
    content = load_url(url)
    return simplejson.loads(content)

def load_xml(url):
    content = load_url(url)
    return gnosis.xml.objectify.make_instance(content)

def load_gdata(url):
    content = load_url(url)
    return gdata.GDataFeedFromString(content)

# --------------------------------------------------------------------
# Generic calls

# http://www.flickr.com/services/api/
def flickr_call(method, **kw):
    url = 'http://api.flickr.com/services/rest/'
    args = {
        'api_key': apikey.flickr,
        'method': method,
        'format': 'json',
        'nojsoncallback': 1,
    }
    args.update(kw)
    payload = urllib.urlencode(args)
    url = url + '?' + payload
    return load_json(url)

# http://www.last.fm/api/intro
def lastfm_call(method, **kw):
    url = 'http://ws.audioscrobbler.com/2.0/'
    args = {
        'api_key': apikey.lastfm,
        'method': method,
    }
    args.update(kw)
    payload = urllib.urlencode(args)
    url = url + '?' + payload
    return load_xml(url)

# http://codian.springnote.com/pages/89009
def me2day_call(method, id, **kw):
    url = 'http://me2day.net/api/'
    url = url + method + '/' + id + '.json'
    if kw:
        payload = urllib.urlencode(kw)
        url = url + '?' + payload
    return load_json(url)

# http://groups.google.com/group/twitter-development-talk/web/api-documentation
def twitter_call(method, id, **kw):
    url = 'http://twitter.com/statuses/'
    url = url + method + '/' + id + '.json'
    if kw:
        payload = urllib.urlencode(kw)
        url = url + '?' + payload
    return load_json(url)

# http://code.google.com/apis/youtube/reference.html
def youtube_call(cls, method, id, **kw):
    url = 'http://gdata.youtube.com/feeds/api/'
    url = url + cls + '/' + id + '/' + method
    if kw:
        payload = urllib.urlencode(kw)
        url = url + '?' + payload
    return load_gdata(url)

# --------------------------------------------------------------------
# Get friends

def flickr_get_friends(nsid):
    response = flickr_call('flickr.contacts.getPublicList', user_id=nsid)
    friends = [contact['nsid'] for contact in response['contacts']['contact']]
    return friends

def lastfm_get_friends(id):
    response = lastfm_call('user.getFriends', user=id)
    friends = [user.name.PCDATA for user in response.friends.user]
    return friends

def me2day_get_friends(id):
    response = me2day_call('get_friends', id)
    friends = [friend['id'] for friend in response['friends']]
    return friends

def twitter_get_friends(id):
    response = twitter_call('friends', id, lite='true')
    friends = [friend['screen_name'] for friend in response]
    return friends

def youtube_get_friends(id):
    yt = 'http://gdata.youtube.com/schemas/2007'
    response = youtube_call('users', 'subscriptions', id)
    friends = [entry.FindExtensions('username', yt)[0].text for entry in response.entry]
    return friends
